#include "stdafx.h"
#include "windows.h"
#include <stdio.h>
#include "nativeAPI.h"

BOOL EnablePrivilege(PCSTR name)
{
    TOKEN_PRIVILEGES priv = {1, {0, 0, SE_PRIVILEGE_ENABLED}};
    LookupPrivilegeValue(0, name, &priv.Privileges[0].Luid);

    HANDLE hToken;
    OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES, &hToken);

    AdjustTokenPrivileges(hToken, FALSE, &priv, sizeof priv, 0, 0);
    BOOL rv = GetLastError() == ERROR_SUCCESS;

    CloseHandle(hToken);
    return rv;
}


int main(int argc, char* argv[])
{
    ULONG pid;
	HANDLE hProcess, hCurrentProcess;
	SYSTEM_HANDLE_INFORMATION  *aHandles;
	ULONG  nHandles, nCount;
	ULONG  *pULONG;
	
	//1. Parameter parsing
    if (argc == 1) 
	{
		printf("HELP : ListHandles PID\n");
		return 0;
	}
	pid = strtoul(argv[1], 0, 0);

	//2. Get process handle
    EnablePrivilege(SE_DEBUG_NAME);
    hProcess = OpenProcess(PROCESS_DUP_HANDLE, FALSE, pid);
	hCurrentProcess = GetCurrentProcess();

	//3. Get Handle's Information
    nCount = 100;
    pULONG = (PULONG)malloc(nCount * sizeof(SYSTEM_HANDLE_INFORMATION) + sizeof(ULONG));
    while (ZwQuerySystemInformation(SystemHandleInformation, pULONG, 
				nCount * sizeof(SYSTEM_HANDLE_INFORMATION)+ sizeof(ULONG)
				, 0) == STATUS_INFO_LENGTH_MISMATCH)
	{
		free(pULONG);
		nCount += 50;
		pULONG = (PULONG)malloc(nCount * sizeof(SYSTEM_HANDLE_INFORMATION) + sizeof(ULONG));
	}
	nHandles = *pULONG;
	aHandles = (PSYSTEM_HANDLE_INFORMATION)(pULONG + 1);
	
	//4. Print Handle's Information
	printf("Process ID : %x\n", pid);
    for (ULONG i = 0; i < nHandles; i++) 
	{
        if (aHandles[i].ProcessId == pid) 
		{
			HANDLE hObject;
			OBJECT_BASIC_INFORMATION obi;
			POBJECT_TYPE_INFORMATION pOti;
			POBJECT_NAME_INFORMATION pOni;
			ULONG nTypeName, nObjectName, n;

			if(DuplicateHandle(hProcess, (HANDLE)aHandles[i].Handle,
				hCurrentProcess, &hObject, 0, 0, DUPLICATE_SAME_ACCESS) == FALSE)
				continue;

            ZwQueryObject(hObject, ObjectBasicInformation, &obi, sizeof(obi), &n);
            printf("%p %04hx %3lx %3ld %4ld ", 
                   aHandles[i].Object, aHandles[i].Handle, obi.Attributes,
                   obi.HandleCount - 1, obi.PointerCount - 2);
			//Object Type
            nTypeName = obi.TypeInformationLength + 2;
            pOti = (POBJECT_TYPE_INFORMATION)malloc(nTypeName);

            ZwQueryObject(hObject, ObjectTypeInformation, pOti, nTypeName, &nTypeName);
            printf("%-14.*ws ", pOti->Name.Length / 2, pOti->Name.Buffer);
			//Object Name
            nObjectName = obi.NameInformationLength == 0 
                ? MAX_PATH * sizeof (WCHAR) : obi.NameInformationLength;

             pOni = (POBJECT_NAME_INFORMATION)malloc(nObjectName);
            
            if (NT_SUCCESS(ZwQueryObject(hObject, ObjectNameInformation, pOni,
							nObjectName, &nObjectName)))
                printf("%.*ws", pOni->Name.Length / 2, pOni->Name.Buffer);

            printf("\n");
			
			free(pOni);	free(pOti); CloseHandle(hObject);
        }
    }
    free(aHandles);
    CloseHandle(hProcess);

    return 0;
}

