//Sound.c
#include "ntddk.h"
#include "Prism.h"
#include "Intel80x86.h"
#include "PageFault.h"


typedef struct _INFO_PAGEFAULT
{
	ULONG nOccurCode;
	ULONG nAccessMemory;
	ULONG nErrorCode;
	ULONG nPID;
}INFO_PAGEFAULT, *PINFO_PAGEFAULT;

#define MAX_ARRAY		100
//
INFO_PAGEFAULT	G_aInfoPageFault[MAX_ARRAY];
ULONG			G_nWriteIndex = -1;
ULONG			G_nReadIndex = -1;
KEVENT			g_eventMon;

PKTHREAD		G_pThread = NULL;
BOOLEAN			G_bEnd = FALSE;


//
INT_API IntHandler_PageFault(ULONG nOccurCode, ULONG nAccessMemory, ULONG nErrorCode)
{	
	ULONG nWriteIndex;
	
	if((PVOID)PsGetCurrentThread() != (PVOID)G_pThread)
	{
		nWriteIndex = G_nWriteIndex;
		nWriteIndex ++;
		if(nWriteIndex >= MAX_ARRAY)
			nWriteIndex = 0;
		
		G_aInfoPageFault[nWriteIndex].nOccurCode = nOccurCode;
		G_aInfoPageFault[nWriteIndex].nAccessMemory = nAccessMemory;	
		G_aInfoPageFault[nWriteIndex].nErrorCode = nErrorCode;	
		G_aInfoPageFault[nWriteIndex].nPID = (ULONG)PsGetCurrentProcessId();
		
		G_nWriteIndex = nWriteIndex;
		KeSetEvent(&g_eventMon, IO_NO_INCREMENT, FALSE);
	}
}


BEGIN_MAKE_INT_HANDLER()
	MAKE_INT_HANDLER14_PAGEFAULT(IntHandler_PageFault)
END_MAKE_INT_HANDLER()	


VOID ThreadProc( PVOID pParam )
{
	while(!G_bEnd)
	{
		KeWaitForSingleObject( &g_eventMon, Executive, KernelMode, FALSE, NULL );
		
		while(G_nReadIndex != G_nWriteIndex)
		{
			G_nReadIndex ++;
			if(G_nReadIndex >= MAX_ARRAY)
				G_nReadIndex = 0;			

			DbgPrint("ProcessID : %04d Memory(0x%08x) ", 
			G_aInfoPageFault[G_nReadIndex].nPID, 
			G_aInfoPageFault[G_nReadIndex].nAccessMemory);

			if(G_bEnd) break;
		}
		
		KeResetEvent(&g_eventMon);
	}
}


void StartMonitoring()
{
	HANDLE hEventHandle;
	HANDLE hThread;	
	
	KeInitializeEvent( &g_eventMon, NotificationEvent, FALSE );	
	
	PsCreateSystemThread( &hThread, THREAD_ALL_ACCESS, NULL, NULL, NULL, (PKSTART_ROUTINE) ThreadProc, 0 );
	ObReferenceObjectByHandle( hThread, THREAD_ALL_ACCESS, NULL, KernelMode, (PVOID*) &G_pThread, NULL );
	ZwClose( hThread );
		
	HOOK_INTERRUPT14_PAGE_FAULT();
}

void	EndMonitoring()
{
	UNHOOK_INTERRUPT14_PAGE_FAULT();
	
	KeSetEvent(&g_eventMon, IO_NO_INCREMENT, FALSE);
	
	G_bEnd = TRUE;
		
	if(G_pThread)
	{
		KeWaitForSingleObject( G_pThread, Executive, KernelMode, FALSE, NULL );
		ObDereferenceObject(G_pThread);
		G_pThread = NULL;
	}
}

NTSTATUS My_IoControl(PFILE_OBJECT pFileObject, ULONG nIoCode, PCHAR pSystemBuffer, ULONG nInput, ULONG nOutput, ULONG *nReturnOut)
{
	if(nIoCode == IOCTL_START_MON)
	{
		StartMonitoring();
	}	
	else if(nIoCode == IOCTL_END_MON)
	{
		EndMonitoring();
	}

	return STATUS_SUCCESS;
}

VOID My_DriverEntry(IN PDRIVER_OBJECT DriverObject ,PDEVICE_OBJECT deviceObject)
{
}

VOID My_DriverUnload(IN PDRIVER_OBJECT DriverObject)
{
		EndMonitoring();
}
