#ifndef _INTEL80X86_H
#define _INTEL80X86_H
//intel80x86.h

#define LOWORD(i)           ((unsigned short)(unsigned int)(i))
#define HIWORD(i)           ((unsigned short)((((unsigned int)(i)) >> 16) & 0xFFFF))

typedef UCHAR               BYTE,  *PBYTE,  **PPBYTE;
typedef USHORT              WORD,  *PWORD,  **PPWORD;
typedef ULONG               DWORD, *PDWORD, **PPDWORD;

typedef int                 BOOL,  *PBOOL,  **PPBOOL;
typedef void                *PVOID,  **PPVOID;

typedef long		        NTSTATUS, *PNTSTATUS, **PPNTSTATUS;
        

/* UNICODE (wide character) types */
#ifndef __cplusplus
typedef short unsigned int wchar_t;
#endif	/* __cplusplus */
typedef wchar_t WCHAR;
typedef WCHAR *PWCHAR;
typedef WCHAR *LPWCH, *PWCH;
typedef CONST WCHAR *LPCWCH, *PCWCH;
typedef WCHAR *NWPSTR;
typedef WCHAR *LPWSTR, *PWSTR;
typedef CONST WCHAR *LPCWSTR, *PCWSTR;

typedef unsigned __int64    QWORD, *PQWORD, **PPQWORD;
typedef LARGE_INTEGER
        PHYSICAL_ADDRESS, *PPHYSICAL_ADDRESS, **PPPHYSICAL_ADDRESS;

typedef struct _BIT
{
	union
	{
		struct
		{
			DWORD dValue;
		};
		struct
		{
			int bit1	:	1; int bit2	:	1; int bit3	:	1; int bit4	:	1;
			int bit5	:	1; int bit6	:	1; int bit7	:	1; int bit8	:	1;
			int bit9	:	1; int bit10:	1; int bit11:	1; int bit12:	1;
			int bit13	:	1; int bit14:	1; int bit15:	1; int bit16:	1;
			int bit17	:	1; int bit18:	1; int bit19:	1; int bit20:	1;
			int bit21	:	1; int bit22:	1; int bit23:	1; int bit24:	1;
			int bit25	:	1; int bit26:	1; int bit27:	1; int bit28:	1;
			int bit29	:	1; int bit30:	1; int bit31:	1; int bit32:	1;
		};
	};
}BIT, *PBIT;

typedef struct _SEG_DESCRIPTOR
{
    union
    {
        struct
        {
            DWORD dwValueLow; 
            DWORD dwValueHigh;
        };
        struct
        {
            unsigned short  Limit_0_15	 :16;
            unsigned short  Base_0_15	 :16;
            unsigned char   Base_16_23	 : 8;
            
			unsigned char    Accessed    : 1;
            unsigned char	 Type	     : 3; // S Flag   ..
            unsigned char    S			 : 1; //1 = General Segment, 0 = System
            unsigned char    DPL         : 2; // descriptor privilege level
            unsigned char    Present     : 1; // segment present
            
		    unsigned char    Limit_16_19 : 4;
		    unsigned char    Unused      : 1;
		    unsigned char    Reserved    : 1;
		    unsigned char    DefaultBit  : 1;// 0=16-bit, 1=32-bit		    
		    unsigned char    Granularity : 1;// 0=1Byte, 1=4KB
		    
		    unsigned char    Base_24_31	 : 8;
        };
    };
}SEG_DESCRIPTOR, *PSEG_DESCRIPTOR;


typedef struct _CODE_SEG_DESCRIPTOR
{
    union
    {
        struct
        {
            DWORD dwValueLow; 
            DWORD dwValueHigh;
        };
        struct
        {	
		    unsigned short  Limit_0_15	 :16;
		    unsigned short  Base_0_15	 :16;
		    unsigned char   Base_16_23	 : 8;
		
		    unsigned char    Accessed    : 1;
		    unsigned char    Read	     : 1;//0 : OnlyExecute 1:Read/Execute
		    unsigned char    Conforming  : 1;//1 : Conforming
		    unsigned char    Code_data   : 1;//1 : Code, 0:Data
		    unsigned char    S			 : 1;
		    unsigned char    DPL         : 2;
		    unsigned char    Present     : 1;
		
		    unsigned char    Limit_16_19 : 4;
		    unsigned char    Unused      : 1;
		    unsigned char    Reserved    : 1;
		    unsigned char    DefaultBit  : 1;// 0=16-bit, 1=32-bit	
		    unsigned char    Granularity : 1;// 0=1Byte, 1=4KB
		
		    unsigned char    Base_24_31	 : 8;
		};
	};	
}CODE_SEG_DESCRIPTOR, *PCODE_SEG_DESCRIPTOR;

typedef struct _DATA_SEG_DESCRIPTOR
{
    union
    {
        struct
        {
            DWORD dwValueLow; 
            DWORD dwValueHigh;
        };
        struct
        {	
		    unsigned short  Limit_0_15	 :16;
		    unsigned short  Base_0_15	 :16;
		    unsigned char   Base_16_23	 : 8;
		
		    unsigned char    Accessed    : 1;
		    unsigned char    Write	     : 1; //0 : ReadOnly, 1:Read/Write
		    unsigned char    ExpandDown  : 1; //0 : ExpandUp(Normal), 1:ExpandDown
		    unsigned char    Code_data   : 1; //0 : Data, 1:Code
		    unsigned char    S			 : 1;
		    unsigned char    DPL         : 2;
		    unsigned char    Present     : 1;
		
		    unsigned char    Limit_16_19 : 4;
		    unsigned char    Unused      : 1;
		    unsigned char    Reserved    : 1;
		    unsigned char    DefaultBit   : 1; //0=16-bit, 1=32-bit
		    unsigned char    Granularity : 1;  //0=1Byte, 1 = 4KByte
		
		    unsigned char    Base_24_31	 : 8;
		};
	};	
}DATA_SEG_DESCRIPTOR, *PDATA_SEG_DESCRIPTOR;

typedef struct _CALLGATE_DESCRIPTOR
{
    union
    {
        struct
        {
            DWORD dwValueLow; 
            DWORD dwValueHigh;
        };
        struct
        {		
		    unsigned short  Offset_0_15 : 16;
		    unsigned short  Selector	: 16;
		
		    unsigned char    Param_count : 4;
		    unsigned char    Dummy_bits  : 4;
		
		    unsigned char    Type        : 4;
		    unsigned char    S			 : 1;
		    unsigned char    DPL         : 2;
		    unsigned char    Present     : 1;
		
		    unsigned short   Offset_16_31 : 16;
		};
	};	
}CALLGATE_DESCRIPTOR, *PCALLGATE_DESCRIPTOR;


typedef struct _TASKGATE_DESCRIPTOR
{
    union
    {
        struct
        {
            DWORD dwValueLow; 
            DWORD dwValueHigh;
        };
        struct
        {		
		    unsigned short  Available0 	: 16;
		    unsigned short  Selector	: 16;
		
		    unsigned char    Available1 : 8;
		
		    unsigned char    Type        : 5; //00101
		    unsigned char    DPL         : 2;
		    unsigned char    Present     : 1;
		
		    unsigned short   Available2	 : 16;
		};
	};	
}TASKGATE_DESCRIPTOR, *PTASKGATE_DESCRIPTOR;


//EFlags Register
typedef struct _X86_EFLAGS	//EFLAGS Register
{
	union
	{
		struct
		{
			DWORD dValue;            // packed value
		};
		struct
		{
			unsigned CF       :  1; //Carry Flag
			unsigned Default1 :  1; //1
			unsigned PF       :  1; //Parity Flag
			unsigned Default2 :  1; //0
			unsigned AF       :  1; //Auxiliary Carry Flag
			unsigned Default3 :  1; //0
			unsigned ZF       :  1; //Zero Flag
			unsigned SF		  :  1; //Sign Flag  <<   Status Flag

			unsigned TF       :  1; //Trap Flag
			unsigned IF       :  1; //Interrupt enable Flag
			unsigned DF       :  1; //Direction Flag
			unsigned OF       :  1; //Overflow Flag
			unsigned IOPL     :  2; //I/O Privilege Level
			unsigned NT       :  1; //Nested Task
			unsigned Default4 :  1; //0

			unsigned RF       :  1; //Resume Flag
			unsigned VM		  :  1; //Virtual 8086 Mode
			unsigned AC       :  1; //Alignment Check
			unsigned VIF      :  1; //Virtual Interrupt Flag
			unsigned VIP      :  1; //Virtual Interrupt Pending
			unsigned ID       :  1; //ID Flag
			//unsigned Reserve   : 10;
		};
	};
}X86_EFLAGS, *PX86_EFLAGS;

#pragma pack(1)
typedef struct _IDTR {
        short Limit;
        unsigned int Base;
} IDTR, *PIDTR;

typedef struct _GDTR {
        short Limit;
        unsigned int Base;
} GDTR, *PGDTR;


typedef struct _X86_CR3 
{
	union
	{
    	struct
		{
        	DWORD dValue;            // packed value
		};
    	struct
		{
			unsigned Reserved1 :  3;
			unsigned PWT       :  1; // page-level write-through
			unsigned PCD       :  1; // page-level cache disabled
			unsigned Reserved2 :  7;
			unsigned PFN       : 20; // page-frame number
		};
	};
}X86_CR3, *PX86_CR3;


typedef struct _X86_SELECTOR
{
	union
    {
	    struct
        {
	        WORD wValue;            //
	        WORD wReserved;
        };
	    struct
        {
	        unsigned RPL      :  2; //
	        unsigned TI       :  1; //
	        unsigned Index    : 13; //
	        unsigned Reserved : 16;
        };
    };
}
X86_SELECTOR, *PX86_SELECTOR;

typedef struct _IDT_ENTRY {
        unsigned short	Offset_0_15;
        unsigned short	Selector;
        unsigned char	Reserved;
        unsigned char	Type:4;  //1111(Trap Gate), 1110(Interrupt Gate)
        unsigned char 	Always0:1;	
        unsigned char 	DPL:2;
        unsigned char 	Present:1;
        unsigned short 	Offset_16_31;
} IDT_ENTRY, *PIDT_ENTRY;

#pragma pack()

typedef struct _X86_TSS
{
	unsigned short	Back_Link,__blh;
	unsigned long	ESP0;
	unsigned short	SS0,__SS0h;
	unsigned long	ESP1;
	unsigned short	SS1,__SS1h;
	unsigned long	ESP2;
	unsigned short	SS2,__SS2h;
	unsigned long	__CR3;
	unsigned long	EIP;
	unsigned long	EFlags;
	unsigned long	EAX,ECX,EDX,EBX;
	unsigned long	ESP;
	unsigned long	EBP;
	unsigned long	ESI;
	unsigned long	EDI;
	unsigned short	ES, __ESh;
	unsigned short	CS, __CSh;
	unsigned short	SS, __SSh;
	unsigned short	DS, __DSh;
	unsigned short	FS, __FSh;
	unsigned short	GS, __GSh;
	unsigned short	LDT, __LDTh;
	
	unsigned short	Trace, OffsetIOBitmap;
	
	//...
	//unsigned long	io_bitmap[IO_BITMAP_SIZE+1];
}X86_TSS, *PX86_TSS;


typedef struct _X86_PDE
{
union
	{
	struct
		{
        	DWORD dwValue;            // packed value
		};
    	struct
		{
			unsigned P         :  1; // present (1 = present)
			unsigned RW        :  1; // read/write
			unsigned US        :  1; // user/supervisor
			unsigned PWT       :  1; // page-level write-through
			unsigned PCD       :  1; // page-level cache disabled
			unsigned A         :  1; // accessed
			unsigned Reserved  :  1; // dirty
			unsigned PS        :  1; // page size (0 = 4-KB page)
			unsigned G         :  1; // global page
			unsigned Available :  3; // available to programmer
			unsigned PFN       : 20; // page-frame number
		};
	};
}X86_PDE, *PX86_PDE;


typedef struct _X86_PTE
{
	union
    {
    	struct
        {
        	DWORD dwValue;            // packed value
        };
    	struct
        {
	        unsigned P         :  1; // present (1 = present)
	        unsigned RW        :  1; // read/write
	        unsigned US        :  1; // user/supervisor
	        unsigned PWT       :  1; // page-level write-through
	        unsigned PCD       :  1; // page-level cache disabled
	        unsigned A         :  1; // accessed
	        unsigned D         :  1; // dirty
	        unsigned Unused    :  1; // 0
	        unsigned G         :  1; // global page
	        unsigned Available :  3; // available to programmer
	        unsigned PFN       : 20; // page-frame number
        };
    };
}X86_PTE, *PX86_PTE;


#define	GATE_TYPE_TRAP		0xF
#define GATE_TYPE_INT		0xE

typedef struct _X86_GATE
{
	union
    {
    	struct
        {
        	DWORD dValueLow;          // packed value
        	DWORD dValueHigh;
        };
    	struct
    	{
	        unsigned OffsetLow  : 16; // bits 15..00
	        unsigned Selector   : 16; // segment selector
	        unsigned Parameters :  5; // parameters
	        unsigned Reserved   :  3;
	        unsigned Type       :  4; // 1110 : ͷƮ Ʈ , 1111 : ƮƮ
	        unsigned S          :  1; // always 0
	        unsigned DPL        :  2; // descriptor privilege level
	        unsigned P          :  1; // segment present
	        unsigned OffsetHigh : 16; // bits 31..16
	    };
    };
}X86_GATE, *PX86_GATE, **PPX86_GATE;

#endif
